from django.utils.translation import gettext_lazy as _

from wagtail.admin.edit_handlers import FieldPanel


class ShowSimilarPanel(FieldPanel):
    def __init__(self, field_name, score_threshold=20, max_items=10, **kwargs):
        self.threshold = score_threshold
        self.max_items = max_items
        super().__init__(field_name, **kwargs)

    def clone_kwargs(self):
        kwargs = super().clone_kwargs()
        kwargs.update({
            "score_threshold": self.threshold,
            "max_items": self.max_items,
        })

        return kwargs

    def get_bound_panel(self, instance=None, request=None, form=None, prefix="panel"):
        bound_panel = super().get_bound_panel(instance, request, form, prefix)
        bound_panel.threshold = self.threshold
        bound_panel.max_items = self.max_items

        return bound_panel

    class BoundPanel(FieldPanel.BoundPanel):
        template_name = "wagtailshowsimilar/field_panel.html"

        def get_context_data(self, parent_context=None):
            context = super().get_context_data(parent_context)

            meta = getattr(self.instance, "_meta")
            context.update({
                "label": _("Similar items:"),
                "target_model": f"{meta.app_label}.{meta.model_name}",
                "instance_id": self.instance.id if self.instance else None,
                "threshold": self.threshold,
                "max_items": self.max_items,
            })

            return context

        class Media:
            js = ("wagtailshowsimilar/showsimilar.js", )



from django.apps import apps
from django.views.decorators.http import require_GET
from django.http import JsonResponse
from django.urls import reverse

from wagtail.search.backends import get_search_backend
from wagtail.core.models import Page

backend = get_search_backend()


@require_GET
def search(request):
    search_query = request.GET.get("query")
    model = apps.get_model(request.GET.get("model"))
    field = request.GET.get("field")
    try:
        instance_id = int(request.GET.get("instance_id"))
    except ValueError:
        instance_id = None
    threshold = float(request.GET.get("threshold"))
    max_items = int(request.GET.get("max_items"))

    query = backend.search(search_query, model, fields=[field])
    try:
        q = query.annotate_score("_score")
        # Since django queryset is lazy, this will trigger queryset to
        # be evaluated to make sure whether annotate_score is supported
        # by current search backend.
        _ = q[0]
        query = q
        with_score = True
    except:
        with_score = False

    response = {"items": [], "is_trimmed": False}
    for q in query:
        if (with_score and q._score > threshold) or not with_score:
            if len(response["items"]) < max_items:
                if q.id != instance_id:
                    if hasattr(q, "get_showsimilar_url"):
                        url = getattr(q, "get_showsimilar_url")()
                    elif isinstance(q, Page):
                        url = reverse(
                            "wagtailadmin_pages:edit",
                            args=(q.page_ptr.id,)
                        )
                    else:
                        url = None
                    response["items"].append({
                        "value": getattr(q, field),
                        "url": url,
                    })
            else:
                response["is_trimmed"] = True
                break
        else:
            break

    return JsonResponse(response)



from django.urls import path
try:
    from wagtail.admin.decorators import require_admin_access
except ImportError:
    from wagtail.admin.auth import require_admin_access

from wagtailshowsimilar.views import search

urlpatterns = [
    path(r"search/", require_admin_access(search)),
]
